import os, sys
basepath = os.getcwd()
class VRMDataFemale:
    path = "jane2-dress.vrm"
    outfile = "vrm-vroid-normal-female.blend"
    editfile = "modelling-vroid-normal-female.blend"
    armature_name = "female"
    mixamo_animation_path = basepath + "/assets/blender/mixamo/female/"
    mixamo_animations = ["idle", "walking", "running", "jump", "left_turn", "right_turn", "swimming", "treading_water", "sitting", "hanging-climb", "hanging-idle"]
    fbx_scale = 0.89
class VRMDataMale:
    path = "buch1.vrm"
    outfile = "vrm-vroid-normal-male.blend"
    editfile = "modelling-vroid-normal-male.blend"
    armature_name = "male"
    mixamo_animation_path = basepath + "/assets/blender/mixamo/male/"
    mixamo_animations = ["idle", "walking", "running", "jump", "left_turn", "right_turn", "swimming", "treading_water", "sitting", "hanging-climb", "hanging-idle"]
    fbx_scale = 1.0
class VRMDataMaleBabyShape:
    path = "buch1-chibi.vrm"
    outfile = "shapes/male/vrm-vroid-normal-male-chibi.blend"
    editfile = "modelling-shape-male-chibi.blend"
    armature_name = "male"
    mixamo_animation_path = basepath + "/assets/blender/mixamo/male/"
    mixamo_animations = []
    fbx_scale = 1.0

class ExportMappingFemale:
    blend_path = "assets/blender/" + "vrm-vroid-normal-female.blend"
    gltf_path = "assets/blender/" + "characters/female/vroid-normal-female.gltf"
    ogre_scene = "characters/female/vroid-normal-female.scene"
    inner_path = "Object"
    objs = ["female", "Body", "Hair", "Face"]
    armature_name = "female"
    outfile = "tmp-female.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

class ExportMappingMale:
    blend_path = "assets/blender/" + "vrm-vroid-normal-male.blend"
    gltf_path = "assets/blender/" + "characters/male/vroid-normal-male.gltf"
    ogre_scene = "characters/male/vroid-normal-male.scene"
    inner_path = "Object"
    objs = ["male", "Body", "Hair", "Face"]
    armature_name = "male"
    outfile = "tmp-male.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

class ExportMappingMaleEdited:
    blend_path = "assets/blender/characters/" + "edited-normal-male.blend"
    gltf_path = "characters/male/normal-male.gltf"
    ogre_scene = "characters/male/vroid-normal-male.scene"
    inner_path = "Object"
    objs = ["male", "Body", "Hair", "Face", "BackHair"]
    armature_name = "male"
    outfile = "tmp-male.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

class ExportMappingMaleClothedEdited:
    blend_path = "assets/blender/characters/" + "edited-normal-male.blend"
    gltf_path = "characters/male/normal-male.gltf"
    ogre_scene = "characters/male/vroid-normal-male.scene"
    inner_path = "Object"
    objs = ["male", "Body", "Hair", "Face", "BackHair", "Tops", "Bottoms", "Shoes", "Accessory"]
    armature_name = "male"
    outfile = "tmp-male.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

class ExportMappingFemaleEdited:
    blend_path = "assets/blender/characters/" + "edited-normal-female.blend"
    gltf_path = "characters/female/normal-female.gltf"
    ogre_scene = "characters/female/vroid-normal-female.scene"
    inner_path = "Object"
#    objs = ["male", "Body", "Hair", "Face", "BackHair", "Tops", "Bottoms", "Shoes", "Accessory"]
    objs = ["female", "Body", "Hair", "Face", "BackHair", "Tops", "Bottoms", "Shoes", "Accessory"]
    armature_name = "female"
    outfile = "tmp-female.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

class ExportMappingMaleBabyShape:
    blend_path = "assets/blender/shapes/male/" + "vrm-vroid-normal-male-chibi.blend"
    gltf_path = "assets/blender/" + "characters/shapes/male/chibi/vroid-normal-male-chibi.gltf"
    ogre_scene = "characters/shapes/male/chibi/vroid-normal-male-chibi.scene"
    inner_path = "Object"
    objs = ["male", "Body"]
    armature_name = "male"
    outfile = "tmp-male-chibi.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

class ExportMappingMaleTestShapeEdited:
    blend_path = "assets/blender/" + "edited-shape-test-male.blend"
    gltf_path = "characters/shapes/male/edited-shape-test-male.gltf"
    ogre_scene = "characters/shapes/male/chibi/edited-shape-test-male.scene"
    inner_path = "Object"
    objs = ["male", "Body"]
    armature_name = "male"
    outfile = "tmp-male-chibi.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

class ExportMappingMaleBaseShapeEdited:
    blend_path = "assets/blender/" + "edited-normal-male-base.blend"
    gltf_path = "characters/shapes/male/edited-normal-male-base.gltf"
    ogre_scene = "characters/shapes/male/chibi/edited-normal-male-base.scene"
    inner_path = "Object"
    objs = ["male", "Body"]
    armature_name = "male"
    outfile = "tmp-male-chibi.blend"
    default_action = 'default'
    def __init__(self):
        self.files = []
        for fobj in self.objs:
            self.files.append({"name": fobj})

